@@ -11,6 +11,8 @@
  * @Short_Description: a grid item.
  *
  * GooCanvasGrid represents a grid item.
+ * A grid consists of a number of equally-spaced horizontal and vertical
+ * grid lines, plus an optional border.
  *
  * It is a subclass of #GooCanvasItemSimple and so inherits all of the style
  * properties such as "stroke-color", "fill-color" and "line-width".
@@ -23,6 +25,28 @@
  *
  * To get or set the properties of an existing #GooCanvasGrid, use
  * g_object_get() and g_object_set().
+ *
+ * The grid's position and size is specified with the #GooCanvasGrid:x,
+ * #GooCanvasGrid:y, #GooCanvasGrid:width and #GooCanvasGrid:height properties.
+ *
+ * The #GooCanvasGrid:x-step and #GooCanvasGrid:y-step properties specify the 
+ * distance between grid lines. The  #GooCanvasGrid:x-offset and
+ * #GooCanvasGrid:y-offset properties specify the distance before the first
+ * grid lines.
+ *
+ * The horizontal or vertical grid lines can be hidden using the
+ * #GooCanvasGrid:show-horz-grid-lines and #GooCanvasGrid:show-vert-grid-lines
+ * properties.
+ *
+ * The width of the border can be set using the #GooCanvasGrid:border-width
+ * property. The border is drawn outside the area specified with the
+ * #GooCanvasGrid:x, #GooCanvasGrid:y, #GooCanvasGrid:width and
+ * #GooCanvasGrid:height properties.
+ *
+ * Other properties allow the colors and widths of the grid lines to be set.
+ * The grid line color and width properties override the standard
+ * #GooCanvasItemSimple:stroke-color and #GooCanvasItemSimple:line-width
+ * properties, enabling different styles for horizontal and vertical grid lines.
  */
 #include <config.h>
 #include <math.h>
@@ -56,12 +80,15 @@ enum {
   /* Convenience properties. */
   PROP_HORZ_GRID_LINE_COLOR,
   PROP_HORZ_GRID_LINE_COLOR_RGBA,
+  PROP_HORZ_GRID_LINE_COLOR_GDK_RGBA,
   PROP_HORZ_GRID_LINE_PIXBUF,
   PROP_VERT_GRID_LINE_COLOR,
   PROP_VERT_GRID_LINE_COLOR_RGBA,
+  PROP_VERT_GRID_LINE_COLOR_GDK_RGBA,
   PROP_VERT_GRID_LINE_PIXBUF,
   PROP_BORDER_COLOR,
   PROP_BORDER_COLOR_RGBA,
+  PROP_BORDER_COLOR_GDK_RGBA,
   PROP_BORDER_PIXBUF
 };
 
@@ -219,6 +246,20 @@ goo_canvas_grid_install_common_properties (GObjectClass *gobject_class)
 						      0, G_MAXUINT, 0,
 						      G_PARAM_READWRITE));
 
+  /**
+   * GooCanvasGrid:horz-grid-line-color-gdk-rgba
+   *
+   * The color to use for the horizontal grid lines, specified as a GdkRGBA.
+   *
+   * Since: 2.0.1
+   */
+  g_object_class_install_property (gobject_class, PROP_HORZ_GRID_LINE_COLOR_GDK_RGBA,
+                                   g_param_spec_boxed ("horz-grid-line-color-gdk-rgba",
+                                                       _("Horizontal Grid Line Color GdkRGBA"),
+                                                       _("The color to use for the horizontal grid lines, specified as a GdkRGBA"),
+                                                       GDK_TYPE_RGBA,
+                                                       G_PARAM_READWRITE));
+
   g_object_class_install_property (gobject_class, PROP_HORZ_GRID_LINE_PIXBUF,
                                    g_param_spec_object ("horz-grid-line-pixbuf",
 							_("Horizontal Grid Line Pixbuf"),
@@ -240,6 +281,20 @@ goo_canvas_grid_install_common_properties (GObjectClass *gobject_class)
 						      0, G_MAXUINT, 0,
 						      G_PARAM_READWRITE));
 
+  /**
+   * GooCanvasGrid:vert-grid-line-color-gdk-rgba
+   *
+   * The color to use for the vertical grid lines, specified as a GdkRGBA.
+   *
+   * Since: 2.0.1
+   */
+  g_object_class_install_property (gobject_class, PROP_VERT_GRID_LINE_COLOR_GDK_RGBA,
+                                   g_param_spec_boxed ("vert-grid-line-color-gdk-rgba",
+                                                       _("Vertical Grid Line Color GdkRGBA"),
+                                                       _("The color to use for the vertical grid lines, specified as a GdkRGBA"),
+                                                       GDK_TYPE_RGBA,
+                                                       G_PARAM_READWRITE));
+
   g_object_class_install_property (gobject_class, PROP_VERT_GRID_LINE_PIXBUF,
                                    g_param_spec_object ("vert-grid-line-pixbuf",
 							_("Vertical Grid Line Pixbuf"),
@@ -261,6 +316,20 @@ goo_canvas_grid_install_common_properties (GObjectClass *gobject_class)
 						      0, G_MAXUINT, 0,
 						      G_PARAM_READWRITE));
 
+  /**
+   * GooCanvasGrid:border-color-gdk-rgba
+   *
+   * The color to use for the border, specified as a GdkRGBA.
+   *
+   * Since: 2.0.1
+   */
+  g_object_class_install_property (gobject_class, PROP_BORDER_COLOR_GDK_RGBA,
+                                   g_param_spec_boxed ("border-color-gdk-rgba",
+                                                       _("Border Color GdkRGBA"),
+                                                       _("The color to use for the border, specified as a GdkRGBA"),
+                                                       GDK_TYPE_RGBA,
+                                                       G_PARAM_READWRITE));
+
   g_object_class_install_property (gobject_class, PROP_BORDER_PIXBUF,
                                    g_param_spec_object ("border-pixbuf",
 							_("Border Pixbuf"),
@@ -312,7 +381,7 @@ goo_canvas_grid_init (GooCanvasGrid *grid)
 
 /**
  * goo_canvas_grid_new:
- * @parent: the parent item, or %NULL. If a parent is specified, it will assume
+ * @parent: (skip): the parent item, or %NULL. If a parent is specified, it will assume
  *  ownership of the item, and the item will automatically be freed when it is
  *  removed from the parent. Otherwise call g_object_unref() to free it.
  * @x: the x coordinate of the left of the grid.
@@ -344,7 +413,7 @@ goo_canvas_grid_init (GooCanvasGrid *grid)
  *                                             NULL);
  * </programlisting></informalexample>
  * 
- * Returns: a new grid item.
+ * Returns: (transfer full): a new grid item.
  **/
 GooCanvasItem*
 goo_canvas_grid_new (GooCanvasItem      *parent,
@@ -477,12 +546,21 @@ goo_canvas_grid_get_common_property (GObject              *object,
     case PROP_HORZ_GRID_LINE_COLOR_RGBA:
       goo_canvas_get_rgba_value_from_pattern (grid_data->horz_grid_line_pattern, value);
       break;
+    case PROP_HORZ_GRID_LINE_COLOR_GDK_RGBA:
+      goo_canvas_get_gdk_rgba_value_from_pattern (grid_data->horz_grid_line_pattern, value);
+      break;
     case PROP_VERT_GRID_LINE_COLOR_RGBA:
       goo_canvas_get_rgba_value_from_pattern (grid_data->vert_grid_line_pattern, value);
       break;
+    case PROP_VERT_GRID_LINE_COLOR_GDK_RGBA:
+      goo_canvas_get_gdk_rgba_value_from_pattern (grid_data->vert_grid_line_pattern, value);
+      break;
     case PROP_BORDER_COLOR_RGBA:
       goo_canvas_get_rgba_value_from_pattern (grid_data->border_pattern, value);
       break;
+    case PROP_BORDER_COLOR_GDK_RGBA:
+      goo_canvas_get_gdk_rgba_value_from_pattern (grid_data->border_pattern, value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -579,6 +657,10 @@ goo_canvas_grid_set_common_property (GObject              *object,
       cairo_pattern_destroy (grid_data->horz_grid_line_pattern);
       grid_data->horz_grid_line_pattern = goo_canvas_create_pattern_from_rgba_value (value);
       break;
+    case PROP_HORZ_GRID_LINE_COLOR_GDK_RGBA:
+      cairo_pattern_destroy (grid_data->horz_grid_line_pattern);
+      grid_data->horz_grid_line_pattern = goo_canvas_create_pattern_from_gdk_rgba_value (value);
+      break;
     case PROP_HORZ_GRID_LINE_PIXBUF:
       cairo_pattern_destroy (grid_data->horz_grid_line_pattern);
       grid_data->horz_grid_line_pattern = goo_canvas_create_pattern_from_pixbuf_value (value);
@@ -592,6 +674,10 @@ goo_canvas_grid_set_common_property (GObject              *object,
       cairo_pattern_destroy (grid_data->vert_grid_line_pattern);
       grid_data->vert_grid_line_pattern = goo_canvas_create_pattern_from_rgba_value (value);
       break;
+    case PROP_VERT_GRID_LINE_COLOR_GDK_RGBA:
+      cairo_pattern_destroy (grid_data->vert_grid_line_pattern);
+      grid_data->vert_grid_line_pattern = goo_canvas_create_pattern_from_gdk_rgba_value (value);
+      break;
     case PROP_VERT_GRID_LINE_PIXBUF:
       cairo_pattern_destroy (grid_data->vert_grid_line_pattern);
       grid_data->vert_grid_line_pattern = goo_canvas_create_pattern_from_pixbuf_value (value);
@@ -605,6 +691,10 @@ goo_canvas_grid_set_common_property (GObject              *object,
       cairo_pattern_destroy (grid_data->border_pattern);
       grid_data->border_pattern = goo_canvas_create_pattern_from_rgba_value (value);
       break;
+    case PROP_BORDER_COLOR_GDK_RGBA:
+      cairo_pattern_destroy (grid_data->border_pattern);
+      grid_data->border_pattern = goo_canvas_create_pattern_from_gdk_rgba_value (value);
+      break;
     case PROP_BORDER_PIXBUF:
       cairo_pattern_destroy (grid_data->border_pattern);
       grid_data->border_pattern = goo_canvas_create_pattern_from_pixbuf_value (value);
@@ -664,15 +754,16 @@ calculate_start_position (gdouble start_pos,
 			  gdouble redraw_start_pos,
 			  gdouble line_width)
 {
-  gdouble n, result;
+  gdouble n = 0.0, result;
 
   /* We want the first position where pos + line_width/2 >= redraw_start_pos.
      i.e. start_pos + (n * step) + (line_width / 2) >= redraw_start_pos,
      or   (n * step) >= redraw_start_pos - start_pos - (line_width / 2),
      or   n >= (redraw_start_pos - start_pos - (line_width / 2) / step). */
-  n = ceil (((redraw_start_pos - start_pos - (line_width / 2.0))) / step);
+  if (step > 0.0)
+    n = ceil (((redraw_start_pos - start_pos - (line_width / 2.0))) / step);
 
-  if (n <= 0)
+  if (n <= 0.0)
     result = start_pos;
   else
     result = start_pos + (n * step);
@@ -734,6 +825,10 @@ paint_vertical_lines (GooCanvasItemSimple   *simple,
       cairo_line_to (cr, x, max_y);
       cairo_stroke (cr);
 
+      /* Avoid an infinite loop. */
+      if (grid_data->x_step <= 0.0)
+        break;
+
       x += grid_data->x_step;
     }
 }
@@ -792,6 +887,10 @@ paint_horizontal_lines (GooCanvasItemSimple   *simple,
       cairo_line_to (cr, max_x, y);
       cairo_stroke (cr);
 
+      /* Avoid an infinite loop. */
+      if (grid_data->y_step <= 0.0)
+        break;
+
       y += grid_data->y_step;
     }
 }
@@ -917,6 +1016,8 @@ goo_canvas_grid_class_init (GooCanvasGridClass *klass)
  * @Short_Description: a model for grid items.
  *
  * GooCanvasGridModel represents a model for grid items.
+ * A grid consists of a number of equally-spaced horizontal and vertical
+ * grid lines, plus an optional border.
  *
  * It is a subclass of #GooCanvasItemModelSimple and so inherits all of the
  * style properties such as "stroke-color", "fill-color" and "line-width".
@@ -933,6 +1034,30 @@ goo_canvas_grid_class_init (GooCanvasGridClass *klass)
  * To respond to events such as mouse clicks on the grid you must connect
  * to the signal handlers of the corresponding #GooCanvasGrid objects.
  * (See goo_canvas_get_item() and #GooCanvas::item-created.)
+ *
+ * The grid's position and size is specified with the #GooCanvasGridModel:x,
+ * #GooCanvasGridModel:y, #GooCanvasGridModel:width and
+ * #GooCanvasGridModel:height properties.
+ *
+ * The #GooCanvasGridModel:x-step and #GooCanvasGridModel:y-step properties
+ * specify the distance between grid lines. The  #GooCanvasGridModel:x-offset
+ * and #GooCanvasGridModel:y-offset properties specify the distance before the
+ * first grid lines.
+ *
+ * The horizontal or vertical grid lines can be hidden using the
+ * #GooCanvasGridModel:show-horz-grid-lines and
+ * #GooCanvasGridModel:show-vert-grid-lines properties.
+ *
+ * The width of the border can be set using the #GooCanvasGridModel:border-width
+ * property. The border is drawn outside the area specified with the
+ * #GooCanvasGridModel:x, #GooCanvasGridModel:y, #GooCanvasGridModel:width and
+ * #GooCanvasGridModel:height properties.
+ *
+ * Other properties allow the colors and widths of the grid lines to be set.
+ * The grid line color and width properties override the standard
+ * #GooCanvasItemModelSimple:stroke-color and
+ * #GooCanvasItemModelSimple:line-width properties, enabling different styles
+ * for horizontal and vertical grid lines.
  */
 
 GooCanvasItemModelIface *goo_canvas_grid_model_parent_iface;
@@ -954,7 +1079,7 @@ goo_canvas_grid_model_init (GooCanvasGridModel *gmodel)
 
 /**
  * goo_canvas_grid_model_new:
- * @parent: the parent model, or %NULL. If a parent is specified, it will assume
+ * @parent: (skip): the parent model, or %NULL. If a parent is specified, it will assume
  *  ownership of the item, and the item will automatically be freed when it is
  *  removed from the parent. Otherwise call g_object_unref() to free it.
  * @x: the x coordinate of the left of the grid.
@@ -986,7 +1111,7 @@ goo_canvas_grid_model_init (GooCanvasGridModel *gmodel)
  *                                                        NULL);
  * </programlisting></informalexample>
  * 
- * Returns: a new grid model.
+ * Returns: (transfer full): a new grid model.
  **/
 GooCanvasItemModel*
 goo_canvas_grid_model_new (GooCanvasItemModel *parent,
