@@ -121,7 +121,7 @@ goo_canvas_item_base_init (gpointer g_iface)
        * @item: the item that received the signal.
        * @target_item: the target of the event.
        * @event: the event data. The x & y fields contain the mouse position
-       *  in the item's coordinate space. The root_x & root_y fields contain
+       *  in the item's coordinate space. The x_root & y_root fields contain
        *  the same coordinates converted to the canvas coordinate space.
        *
        * Emitted when the mouse enters an item.
@@ -146,7 +146,7 @@ goo_canvas_item_base_init (gpointer g_iface)
        * @item: the item that received the signal.
        * @target_item: the target of the event.
        * @event: the event data. The x & y fields contain the mouse position
-       *  in the item's coordinate space. The root_x & root_y fields contain
+       *  in the item's coordinate space. The x_root & y_root fields contain
        *  the same coordinates converted to the canvas coordinate space.
        *
        * Emitted when the mouse leaves an item.
@@ -171,7 +171,7 @@ goo_canvas_item_base_init (gpointer g_iface)
        * @item: the item that received the signal.
        * @target_item: the target of the event.
        * @event: the event data. The x & y fields contain the mouse position
-       *  in the item's coordinate space. The root_x & root_y fields contain
+       *  in the item's coordinate space. The x_root & y_root fields contain
        *  the same coordinates converted to the canvas coordinate space.
        *
        * Emitted when the mouse moves within an item.
@@ -196,7 +196,7 @@ goo_canvas_item_base_init (gpointer g_iface)
        * @item: the item that received the signal.
        * @target_item: the target of the event.
        * @event: the event data. The x & y fields contain the mouse position
-       *  in the item's coordinate space. The root_x & root_y fields contain
+       *  in the item's coordinate space. The x_root & y_root fields contain
        *  the same coordinates converted to the canvas coordinate space.
        *
        * Emitted when a mouse button is pressed in an item.
@@ -221,7 +221,7 @@ goo_canvas_item_base_init (gpointer g_iface)
        * @item: the item that received the signal.
        * @target_item: the target of the event.
        * @event: the event data. The x & y fields contain the mouse position
-       *  in the item's coordinate space. The root_x & root_y fields contain
+       *  in the item's coordinate space. The x_root & y_root fields contain
        *  the same coordinates converted to the canvas coordinate space.
        *
        * Emitted when a mouse button is released in an item.
@@ -397,7 +397,7 @@ goo_canvas_item_base_init (gpointer g_iface)
       /**
        * GooCanvasItem::child-notify
        * @item: the item that received the signal.
-       * @pspec: the #GParamSpec of the changed child property.
+       * @pspec: (type GLib.ParamSpec): the #GParamSpec of the changed child property.
        *
        * Emitted for each child property that has changed.
        * The signal's detail holds the property name. 
@@ -434,7 +434,7 @@ goo_canvas_item_base_init (gpointer g_iface)
        * @item: the item that received the signal.
        * @target_item: the target of the event.
        * @event: the event data. The x & y fields contain the mouse position
-       *  in the item's coordinate space. The root_x & root_y fields contain
+       *  in the item's coordinate space. The x_root & y_root fields contain
        *  the same coordinates converted to the canvas coordinate space.
        *
        * Emitted when a button in the 4 to 7 range is pressed. Wheel mice are
@@ -516,6 +516,15 @@ goo_canvas_item_base_init (gpointer g_iface)
 								 FALSE,
 								 G_PARAM_READWRITE));
 
+      /**
+       * GooCanvasItem:tooltip:
+       *
+       * The tooltip to display for the item, or %NULL to display no tooltip.
+       *
+       * Note that this property has no effect unless the
+       * #GtkWidget:has-tooltip property is set to %TRUE on the #GooCanvas
+       * containing this item.
+       */
       g_object_interface_install_property (g_iface,
 					   g_param_spec_string ("tooltip",
 								_("Tooltip"),
@@ -536,7 +545,7 @@ goo_canvas_item_base_init (gpointer g_iface)
  * 
  * Returns the #GooCanvas containing the given #GooCanvasItem.
  * 
- * Returns: the #GooCanvas.
+ * Returns: (transfer none): the #GooCanvas.
  **/
 GooCanvas*
 goo_canvas_item_get_canvas (GooCanvasItem *item)
@@ -711,8 +720,8 @@ goo_canvas_item_get_n_children (GooCanvasItem       *item)
  * 
  * Gets the child item at the given stack position.
  * 
- * Returns: the child item at the given stack position, or %NULL if @child_num
- * is out of range.
+ * Returns: (transfer none): the child item at the given stack position, or
+ *  %NULL if @child_num is out of range.
  **/
 GooCanvasItem*
 goo_canvas_item_get_child (GooCanvasItem       *item,
@@ -730,7 +739,7 @@ goo_canvas_item_get_child (GooCanvasItem       *item,
  * 
  * Gets the parent of the given item.
  * 
- * Returns: the parent item, or %NULL if the item has no parent.
+ * Returns: (transfer none): the parent item, or %NULL if the item has no parent.
  **/
 GooCanvasItem*
 goo_canvas_item_get_parent  (GooCanvasItem *item)
@@ -923,7 +932,7 @@ goo_canvas_item_lower          (GooCanvasItem *item,
 /**
  * goo_canvas_item_get_transform:
  * @item: an item.
- * @transform: the place to store the transform.
+ * @transform: (out): the place to store the transform.
  * 
  * Gets the transformation matrix of an item.
  * 
@@ -943,7 +952,7 @@ goo_canvas_item_get_transform  (GooCanvasItem   *item,
  * goo_canvas_item_get_transform_for_child:
  * @item: an item.
  * @child: a child of @item.
- * @transform: the place to store the transform.
+ * @transform: (out): the place to store the transform.
  * 
  * Gets the transformation matrix of an item combined with any special
  * transform needed for the given child. These special transforms are used
@@ -972,7 +981,7 @@ goo_canvas_item_get_transform_for_child  (GooCanvasItem  *item,
 /**
  * goo_canvas_item_set_transform:
  * @item: an item.
- * @transform: the new transformation matrix, or %NULL to reset the
+ * @transform: (allow-none): the new transformation matrix, or %NULL to reset the
  *  transformation to the identity matrix.
  * 
  * Sets the transformation matrix of an item.
@@ -988,10 +997,10 @@ goo_canvas_item_set_transform  (GooCanvasItem        *item,
 /**
  * goo_canvas_item_get_simple_transform:
  * @item: an item.
- * @x: returns the x coordinate of the origin of the item's coordinate space.
- * @y: returns the y coordinate of the origin of the item's coordinate space.
- * @scale: returns the scale of the item.
- * @rotation: returns the clockwise rotation of the item, in degrees (0-360).
+ * @x: (out): returns the x coordinate of the origin of the item's coordinate space.
+ * @y: (out): returns the y coordinate of the origin of the item's coordinate space.
+ * @scale: (out): returns the scale of the item.
+ * @rotation: (out): returns the clockwise rotation of the item, in degrees (0-360).
  * 
  * This function can be used to get the position, scale and rotation of an
  * item, providing that the item has a simple transformation matrix
@@ -1204,7 +1213,7 @@ goo_canvas_item_skew_y         (GooCanvasItem *item,
  * Gets the item's style. If the item doesn't have its own style it will return
  * its parent's style.
  * 
- * Returns: the item's style.
+ * Returns: (transfer none): the item's style.
  **/
 GooCanvasStyle*
 goo_canvas_item_get_style      (GooCanvasItem   *item)
@@ -1521,7 +1530,7 @@ goo_canvas_item_request_update  (GooCanvasItem *item)
 /**
  * goo_canvas_item_get_bounds:
  * @item: a #GooCanvasItem.
- * @bounds: a #GooCanvasBounds to return the bounds in.
+ * @bounds: (out): a #GooCanvasBounds to return the bounds in.
  * 
  * Gets the bounds of the item.
  *
@@ -1548,15 +1557,17 @@ goo_canvas_item_get_bounds  (GooCanvasItem   *item,
  *  be used to determine which parts of the item are tested.
  * @parent_is_visible: %TRUE if the parent item is visible (which
  *  implies that all ancestors are also visible).
- * @found_items: the list of items found so far.
+ * @found_items: (element-type GooCanvas.CanvasItem): the list of items found
+ *  so far.
  * 
  * This function is only intended to be used when implementing new canvas
  * items, specifically container items such as #GooCanvasGroup.
  *
  * It gets the items at the given point.
  * 
- * Returns: the @found_items list, with any more found items added onto
- *  the start of the list, leaving the top item first.
+ * Returns: (element-type GooCanvas.CanvasItem) (transfer none): the
+ *  @found_items list, with any more found items 
+ *  added onto the start of the list, leaving the top item first.
  **/
 GList*
 goo_canvas_item_get_items_at (GooCanvasItem  *item,
@@ -1616,7 +1627,7 @@ goo_canvas_item_is_visible  (GooCanvasItem   *item)
  * 
  * Gets the model of the given canvas item.
  * 
- * Returns: the item's model, or %NULL if it has no model.
+ * Returns: (transfer none): the item's model, or %NULL if it has no model.
  **/
 GooCanvasItemModel*
 goo_canvas_item_get_model	  (GooCanvasItem   *item)
@@ -1743,6 +1754,42 @@ goo_canvas_item_get_requested_area (GooCanvasItem    *item,
 }
 
 
+/**
+ * goo_canvas_item_get_requested_area_for_width:
+ * @item: a #GooCanvasItem.
+ * @cr: a cairo context.
+ * @width: the allocated width.
+ * @requested_area: a #GooCanvasBounds to return the requested area in, in the
+ *  parent's coordinate space. If %FALSE is returned, this is undefined.
+ * 
+ * This function is only intended to be used when implementing new canvas
+ * items, specifically layout items such as #GooCanvasTable.
+ *
+ * It gets the requested area of a child item, assuming it is allocated the
+ * given width. This is useful for text items whose requested height may change
+ * depending on the allocated width.
+ * 
+ * Returns: %TRUE if the item's requested area changes due to the new allocated
+ * width.
+ *
+ * Since: 2.0.1
+ **/
+gboolean
+goo_canvas_item_get_requested_area_for_width (GooCanvasItem	*item,
+					      cairo_t          *cr,
+					      gdouble           width,
+					      GooCanvasBounds  *requested_area)
+{
+  GooCanvasItemIface *iface = GOO_CANVAS_ITEM_GET_IFACE (item);
+
+  if (iface->get_requested_area_for_width)
+    return iface->get_requested_area_for_width (item, cr, width, requested_area);
+  else
+    return FALSE;
+}
+
+
+
 /**
  * goo_canvas_item_get_requested_height:
  * @item: a #GooCanvasItem.
@@ -2294,13 +2341,14 @@ goo_canvas_item_class_install_child_property (GObjectClass *iclass,
  * goo_canvas_item_class_find_child_property:
  * @iclass: a #GObjectClass
  * @property_name: the name of the child property to find
- * @returns: the #GParamSpec of the child property or %NULL if @class has no
- *   child property with that name.
  *
  * This function is only intended to be used when implementing new canvas
  * items, specifically layout container items such as #GooCanvasTable.
  *
  * It finds a child property of a canvas item class by name.
+ *
+ * Returns: (type GObject.ParamSpec) (transfer none): the #GParamSpec of the
+ *  child property or %NULL if @class has no child property with that name.
  */
 GParamSpec*
 goo_canvas_item_class_find_child_property (GObjectClass *iclass,
@@ -2317,14 +2365,15 @@ goo_canvas_item_class_find_child_property (GObjectClass *iclass,
 /**
  * goo_canvas_item_class_list_child_properties:
  * @iclass: a #GObjectClass
- * @n_properties: location to return the number of child properties found
- * @returns: a newly allocated array of #GParamSpec*. The array must be 
- *           freed with g_free().
+ * @n_properties: (out): location to return the number of child properties found
  *
  * This function is only intended to be used when implementing new canvas
  * items, specifically layout container items such as #GooCanvasTable.
  *
  * It returns all child properties of a canvas item class.
+ *
+ * Returns: (array length=n_properties) (transfer full): a newly allocated
+ *  array of #GParamSpec*. The array must be freed with g_free().
  */
 GParamSpec**
 goo_canvas_item_class_list_child_properties (GObjectClass *iclass,
